#' ---
#' title: |
#'   医薬品開発のための統計解析 第２部4章3節 \
#'   医薬品開発における共分散分析の例 \
#'   Green2-4-3a.R
#' author: "Masahiko Katase"
#' date: "2021/7/4"
#' ---

#+ message = FALSE
library(readxl)    # read_excel
library(car)       # Anova
library(emmeans)   # emmeans
library(multcomp)  # cld
library(dplyr)     # cace_when

df <- read_excel("Green2-4.xlsx", sheet = "4-kyobunsan2")
df <- data.frame(df)
df$group <- as.factor(df$group)

#'# 表示4.3.1 データと基本統計量（p.152）
nam <- c("control", "D1", "D2", "D3", "D4")

lst_sub <- list()  # 空のリストを作成
lst_sub[[1]] <- subset(df, group == "Control", select = c(y, weight))
lst_sub[[2]] <- subset(df, group == "D1", select = c(y, weight))
lst_sub[[3]] <- subset(df, group == "D2", select = c(y, weight))
lst_sub[[4]] <- subset(df, group == "D3", select = c(y, weight))
lst_sub[[5]] <- subset(df, group == "D4", select = c(y, weight))

# 空のリストを作成、回帰分析の結果を格納
lst_lm <- list() 
for (i in 1:5) lst_lm[[i]] <- lm(y ~ weight, data = lst_sub[[i]])

# 空のリストを作成、回帰分析の結果から切片と傾きを格納
lst_f <- list() 
for (i in 1:5) lst_f[[i]] <- coef(lst_lm[[i]])

# 空のベクトルを作成、相関係数を格納
vt_r <- numeric() 
for (i in 1:5) vt_r[i] <- cor(lst_sub[[i]]$y, lst_sub[[i]]$weight)

tab <- rbind(lst_f[[1]], lst_f[[2]], lst_f[[3]], lst_f[[4]], lst_f[[5]])
rownames(tab) <- nam
cbind(tab, r = vt_r)

# 空の行列を作成、平均を格納
mx_mean <- matrix(0, nrow = 5, ncol = 2) 
for (i in 1:5) mx_mean[i, ] <- colMeans(lst_sub[[i]])
rownames(mx_mean) <- nam
colnames(mx_mean) <- c("y", "weight")
mx_mean

# 空の行列を作成、標準偏差を格納
mx_sd <- matrix(0, nrow = 5, ncol = 2) 
for (i in 1:5) mx_sd[i, ] <- sapply(lst_sub[[i]], sd)
rownames(mx_sd) <- nam
colnames(mx_sd) <- c("y", "weight")
mx_sd

#'# 表示4.3.2 層別散布図、群ごとの回帰直線と平均（p.153）
g <- case_when(df$group == "Control" ~ 1,
               df$group == "D1" ~ 2,
               df$group == "D2" ~ 3,
               df$group == "D3" ~ 4,
               df$group == "D4" ~ 5)
df <- cbind(df, g = g)
plot(x = df$weight, y = df$y, type = "p", col = df$g, pch = df$g, 
     xlim = c(200, 300), ylim = c(6, 14), ylab = "y", xlab = "weight")
for (i in 1:5) {
  abline(lst_lm[[i]], col = i)
  points(mx_mean[i, 2], mx_mean[i, 1], pch = 3 ,cex = 3, col = i) 
}

#'# 標葉4.3.3 傾きの差の検定（p.154）
center <- mean(df$weight)
lm_out1 <- lm(y ~ group * I(weight - center), data = df, 
              contrasts = list(group = "contr.sum"))
Anova(lm_out1, type = 3)

#'# 表示4.3.4 共分散分析の解析結果（p.155）
lm_out2 <- lm(y ~ group + weight, data = df, 
              contrasts = list(group = "contr.sum"))
summary(lm_out2)
Anova(lm_out2, type = 3)

coef_out <- coef(lm_out2) #　パラメータの取り出し
est_a  <- coef_out[1]
est_g1 <- coef_out[2]
est_g2 <- coef_out[3]
est_g3 <- coef_out[4]
est_g4 <- coef_out[5]
est_g5 <- -(est_g1 + est_g2 + est_g3 + est_g4)
est_b  <- coef_out[6]

fn1 <- function(x) est_a + est_g1 + est_b * x
fn2 <- function(x) est_a + est_g2 + est_b * x
fn3 <- function(x) est_a + est_g3 + est_b * x
fn4 <- function(x) est_a + est_g4 + est_b * x
fn5 <- function(x) est_a + est_g5 + est_b * x

# グラフ
plot(x = df$weight, y = df$y, type = "p", col = df$g, pch = df$g, 
     xlim = c(200, 300), ylim = c(6, 14), ylab = "y", xlab = "weight")
curve(fn1, xlim = c(200, 300), ylim = c(6, 14), col = 1, add = TRUE)
curve(fn2, xlim = c(200, 300), ylim = c(6, 14), col = 2, add = TRUE)
curve(fn3, xlim = c(200, 300), ylim = c(6, 14), col = 3, add = TRUE)
curve(fn4, xlim = c(200, 300), ylim = c(6, 14), col = 4, add = TRUE)
curve(fn5, xlim = c(200, 300), ylim = c(6, 14), col = 5, add = TRUE)

#'# 表示4.3.6 各薬剤とコントロールとの比較（p.156）
# ペアごとのStudentのt検定（多重性を考慮しない比較）
emmeans_out1 <- emmeans(lm_out2, specs = "group")
emmeans_out1
pairs(emmeans_out1, adjust = "none")

# Dunett の検定（多重性を考慮した対照群との比較）
emmeans(lm_out2, specs = trt.vs.ctrl ~ "group", ref = 1)
